// FOURYA/XFIT/Koalariet XDD file
// Licence: Lesser GNU Public License 2.1 (LGPL)

// format description: http://www.ccp14.ac.uk/tutorial/xfit-95/dformat.htm

#ifndef XYLIB_XFIT_XDD_H_
#define XYLIB_XFIT_XDD_H_
#include "xylib.h"

namespace xylib {

    class XfitXddDataSet : public DataSet
    {
        OBLIGATORY_DATASET_MEMBERS(XfitXddDataSet)
    };

}
#endif // XYLIB_XFIT_XDD_H_

