// SpecsLab2 xy data file
// Licence: Lesser GNU Public License 2.1 (LGPL)

// implementation based on a few example files only

#ifndef XYLIB_SPECSXY_H_
#define XYLIB_SPECSXY_H_
#include "xylib.h"

namespace xylib {

    class SpecsxyDataSet : public DataSet
    {
        OBLIGATORY_DATASET_MEMBERS(SpecsxyDataSet)
    };

}
#endif // XYLIB_SPECSXY_H_

